@@ -28,19 +28,19 @@ class MIT(score):
         score.__init__(self,*args,**kwds)
         self.alpha=kwds["chi_alpha"]
     
-    def graph_score(self,number_of_potential_parents,gene_vertex,weights_of_parents,number_of_data_points, list_of_parents): # g(Pa)
+    def graph_score(self,number_of_potential_parents,gene_vertex,weights_of_parents,number_of_data_points): # g(Pa)
         
         # chisquare
         #print list_of_parents
             
-        sorted_parents = sorted(list_of_parents, key=lambda gene: gene.n_disc,reverse=True)
+#        sorted_parents = sorted(list_of_parents, key=lambda gene: gene.n_disc,reverse=True)
         #print "parent.size: ", len(list_of_parents)
         sum_of_l_values = 0.0
         
-        for i in range(len(sorted_parents)):            
-            l_i_sigma = (gene_vertex.n_disc - 1)*(sorted_parents[i].n_disc - 1)
-            for j in range(i):
-                l_i_sigma *= sorted_parents[j].n_disc
+        for i,wi in enumerate(weights_of_parents):            
+            l_i_sigma = (gene_vertex.base_weight() - 1)*(wi - 1)
+            for w in weights_of_parents[:i]:
+                l_i_sigma *= w
             chisquare_value = chi2.isf(1-self.alpha, l_i_sigma)
             #print "chisquare:", chisquare_value
             sum_of_l_values += chisquare_value
