@@ -36,9 +36,11 @@ def eval_func(f_args):
     return f_args[0](*f_args[1:])
 
 def learn_x(v):
-    x,self,min_empty,min_optim,scr,verbose,n_min,limit,score=v
+    x,self,min_empty,min_optim,scr,verbose,n_min,limit,fpr_factor,max_tries=v
 
-    selected_data = self.select_1(x)
+    selected_data = self.select_1(x, sloops=scr.sloops)
+    x_priors = {p : self.get_prior(x,p) for p in selected_data.parents}
+    selected_data.weight_parents(x_priors,fpr_factor,max_tries,scr)
 
     if min_empty:
         score_empty=scr.data_score(selected_data.subset([]))+scr.graph_score(len(parents),x,[],len(selected_data))
@@ -50,7 +52,7 @@ def learn_x(v):
     else:
         score_delta=fpconst.PosInf
     (par,sc),minlist = scr.learn_1(selected_data,verbose,n_min,limit,score_max,score_delta)
-    return x,par,minlist,score 
+    return x,par,minlist,sc
 
 
 
@@ -80,11 +82,74 @@ class experiment:
 class dataset_1:
     """Class representing data selected for learning parents of one vertex
     """
-    def __init__(self,data,vertex,parents,weights=None):
+    def __init__(self,data,vertex,parents):
         self.data=data
         self.vertex=vertex
         self.parents=parents
-        self.weights=weights
+        self.weights={}
+        
+    def weight_parents(self,priors,fpr_factor=None,max_tries=None,score=None):
+        """Assigns weights to potential parents
+        Required before learning
+        """
+        if self.parents:
+            if fpr_factor: # calculate weights with fpr procedure
+                n_parents = len(self.parents)
+                n_data = len(self)
+                no_tries = max(100, int(round( log(2+n_parents)*(max(priors.values())/fpr_factor) )))
+                if max_tries>100:
+                    no_tries=min(no_tries,max_tries)
+                    
+                self_empty = self.subset()
+                empty_score = score.data_score(self_empty) + score.graph_score(n_parents,self.vertex,[],n_data)
+                sample_scores = [[score.lower_bound_for_data_score(self_empty)]*n_parents]
+                
+                # no_tries times shuffle vertex data and add scores to the distributions
+                v_data = [d for d in self.data[-1]]  # create a copy before shuffling
+                for i in range(no_tries):
+                    shuffle(self.data[-1])
+                    sample_scores.append([score.data_score(self.subset([par]))  for par in self.parents])
+                distributions = [sorted(scores) for scores in zip(*sample_scores)]
+                self.data[-1] = v_data
+                
+                for i,par in enumerate(self.parents):
+                    # compute data_score thresholds
+                    thr_ind = no_tries*fpr_factor/priors[par]
+                    ind1 = min(no_tries, int(thr_ind))
+                    ind2 = min(no_tries, ind1+1)
+                    p = thr_ind-ind1
+                    ds_thr = (1-p)*distributions[i][ind1] + p*distributions[i][ind2]
+
+                    # compute required parents' graph_score
+                    g_score = max(0, empty_score-ds_thr)
+                    
+                    # compute parent's weight
+                    r = 2.001
+                    r_score = score.graph_score(n_parents,self.vertex,[r],n_data)
+                    while r_score < g_score:
+                        r*=2
+                        r_score = score.graph_score(n_parents,self.vertex,[r],n_data)
+                    l = r/2
+                    l_score = score.graph_score(n_parents,self.vertex,[l],n_data)
+                    while r-l > 1e-9 and r_score-l_score > 1e-5:
+                        w=(l+r)/2
+                        w_score = score.graph_score(n_parents,self.vertex,[w],n_data)
+                        if w_score < g_score:
+                            l=w
+                            l_score = w_score
+                        else:
+                            r=w
+                            r_score = w_score
+                    self.weights[par]=w
+            
+            else: # calculate weights based on priors only
+                for par in self.parents:
+                    self.weights[par]=par.base_weight()**priors[par]
+            # sort parents and data according to weights
+            order = [i for i,p in sorted(enumerate(self.parents), key = lambda (i,par) : self.weights[par])]
+            self.parents = [self.parents[i] for i in order]
+            self.data = [self.data[i] for i in order]+[self.data[-1]]
+            
         
     def __len__(self):
         return len(self.data[-1])
@@ -94,40 +159,39 @@ class dataset_1:
         """
         res={}
         for i,p in enumerate(self.parents):
-            sign = stats.pearsonr(self.data[i],self.data[-1])[0]
+	    try:
+		sign = stats.pearsonr(self.data[i],self.data[-1])[0]
+	    except ValueError:
+		print self.data[i],self.data[-1]
             if sign >=0:
                 res[p]="+"
             else:
                 res[p]="-"
         return res
         
-    def permute(self):
-        """Randomly permutes child values
-        Warning: modifies data
-        """
-        shuffle(self.data[-1])
-        
     def subset(self,sub=[]):
         """Creates a new dataset_1 object with a subset of parents
-        Warning: does not restrict weights to the subset
         """
         ssub = set(sub)
         sub_ind=[i for i,p in enumerate(self.parents) if p in ssub]
         parents=[self.parents[i] for i in sub_ind]
         data = [self.data[i] for i in sub_ind]
         data.append(self.data[-1])
-        return dataset_1(data,self.vertex,parents,self.weights)
+        d1 = dataset_1(data,self.vertex,parents)
+        if self.weights:
+            d1.weights = {par : self.weights[par] for par in parents}
+        return d1
         
-    def rm_sloops(self):
-        """Removes the child from parent set
-        Warning: modifies data
-        """
-        try:
-            i=self.parents.index(self.vertex)
-            del self.parents[i]
-            del self.data[i]
-        except ValueError:
-            pass
+#    def rm_sloops(self):
+#        """Removes the child from parent set
+#        Warning: modifies data
+#        """
+#        try:
+#            i=self.parents.index(self.vertex)
+#            del self.parents[i]
+#            del self.data[i]
+#        except ValueError:
+#            pass
         
     def stats(self):
         """Counts frequencies of value vectors of
@@ -221,6 +285,9 @@ class gene:
             return self.index==other
         else:
             return self.index==other.index    
+            
+    def base_weight(self):
+        return max(1.5,self.n_disc)
         
 class dataset:
     """
@@ -236,7 +303,6 @@ class dataset:
         self.vertices= []
         self.vertice_names=[]
         self.static=False
-        self.cont_weight=1.5
         self.regulators=[]
         self.parents={}
         self.prior={}
@@ -355,11 +421,15 @@ class dataset:
                 cpd['or'].extend(rec[1:])
             elif rec[0]=="#priorvert": #prior weight of all outgoing edges (except specified in #prioredge command) followed by list of related vertices
                 val=float(rec[1])
+                if val <= 0:
+                    raise Exception("Non-positive prior weight in dataset")
                 for name in rec[2:]:
                     self.prior[name]=val
             elif rec[0]=="#prioredge": #prior weight of edges preceded by edge target (common for all specified edges) and followed by list of edge sources
                 target_name=rec[1]
                 val=float(rec[2])
+                if val <= 0:
+                    raise Exception("Non-positive prior weight in dataset")
                 for name in rec[3:]:
                     self.prior[(target_name,name)]=val
             else:
@@ -498,7 +568,8 @@ class dataset:
         self.n_gen=geneindex
         return self
 
-    def select_1(self,vertex,parents=None):
+
+    def select_1(self,vertex,parents=None, sloops=True):
         """
         given a vertice and a list of his parents,
         select parents values at time t
@@ -506,47 +577,43 @@ class dataset:
         omitting the experiments where the child was perturbed.
         """
         if parents is None:
-            parents = self.get_potential_parents(vertex)
-        #assign weights to parents
-        weights={}
-        for par in parents:
-            weight=float(max(self.cont_weight,par.n_disc))
-            if (vertex.name,par.name) in self.prior.keys():
-                weight=weight**self.prior[(vertex.name,par.name)]
-            elif par.name in self.prior.keys():
-                weight=weight**self.prior[par.name]
-            weights[par]=weight
-        parents.sort(key = lambda p: weights[p])
-
+            parents = self.get_potential_parents(vertex, sloops)
         res = []
         for expment in self.points:
             if vertex not in expment.perturbed:
                 for sample in expment.data:
                     res.append([sample[par] for par in parents]+[sample[vertex+self.n_gen]])
-        return dataset_1(map(list,zip(*res)),vertex,parents,weights)
+        return dataset_1(map(list,zip(*res)),vertex,parents)
 
-    # pbd:debug there are some inconsistencies between this version of getting potential parents and
-    # the one implemented in bnfinder-waszczuk branch - see how it works on input3.txt or input4.txt 
-    def get_potential_parents(self, node):
+
+    def get_potential_parents(self, node, sloops=True):
         """selects potential parents for node"""
-        if self.parents.has_key(node.name):
+        if self.parents.has_key(node.name): # potential parents of a node are explicitly specified
             parents=map(lambda n: self.vertices[self.vertice_names.index(n)],self.parents[node.name])
-        else:
-            #maybe there are regulators specified
-            if self.regulators==[]:
-                parents=[v for v in self.vertices]
-            else:#there is a list of regulators
-                parents=[]
-                for r in self.regulators:
-                    if node.name in r:
-                        break
-                    else:
-                        parents.extend(map(lambda n: self.vertices[self.vertice_names.index(n)],r))
+        elif self.regulators: # regulators are specified
+            parents=[]
+            for r in self.regulators:
+                if node.name in r:
+                    break
+                else:
+                    parents.extend(map(lambda n: self.vertices[self.vertice_names.index(n)],r))
+        else: # all vertices are potential parents
+            parents=[v for v in self.vertices]
+        if not sloops:
+            try:
+                i=parents.index(node)
+                del parents[i]
+            except ValueError:
+                pass
         return parents
 
  
+    def get_prior(self,vertex,parent):
+        return self.prior.get((vertex.name,parent.name),self.prior.get(parent.name,1))
+ 
+ 
     def learn(self,score,data_factor,prior=None,distrib=False,\
-            verbose=None,n_min=1,limit=None,min_empty=None,min_optim=None,fdr_weights=None):
+            verbose=None,n_min=1,limit=None,min_empty=None,min_optim=None,fpr=None,max_tries=None):
         
         scr=score
         
@@ -561,36 +628,43 @@ class dataset:
                 pool=Pool(distrib)
                 mapf=pool.map
 
-
-        score = 0.0
-        pars ={}
-        subpars ={}
-
-
-        # fdr weights have higher priority than self.prior
-        # if both set - ignore later one
-        if self.prior and fdr_weights:
-            print "Warning: fdr parameter set - ignoring priors in dataset"
-
-        if fdr_weights:
-            self.prior = {}
-            for c,p in fdr_weights:
-                self.prior[(c,p)] = log(fdr_weights[(c,p)],max(self.cont_weight,self.vertices[self.vertice_names.index(p)].n_disc))
+        # calculate fpr_factor
+        if fpr:
+            normalizer = 0.
+            n_edge = 0
+            for vertex in self.vertices:
+                for parent in self.get_potential_parents(vertex,scr.sloops):
+                    normalizer += 1./self.get_prior(vertex,parent)
+                    n_edge +=1
+            try:
+                fpr_factor = fpr*n_edge/normalizer
+#                fpr_factor = fpr*self.n_gen/normalizer
+#                fpr_factor = fpr/normalizer
+            except ZeroDivisionError:
+                raise Exception("Potential parents set is empty for each vertex")
+        else:
+            fpr_factor = None
+                
         if distrib:
             from itertools import izip,repeat
-            result=pool.map(learn_x,[(x,self,min_empty,min_optim,scr,verbose,n_min,limit,score) for x in self.vertices])
+            result=pool.map(learn_x,[(x,self,min_empty,min_optim,scr,verbose,n_min,limit,fpr_factor,max_tries) for x in self.vertices])
                             #eval_func,izip([learn_x]*len(self.vertices),self.vertices))
             #pool.close() #raises weird errors...., not necessary in normal bnf operation, since the process exits after learning
         else:
-            result=map(learn_x,[(x,self,min_empty,min_optim,scr,verbose,n_min,limit,score) for x in self.vertices])
+            result=map(learn_x,[(x,self,min_empty,min_optim,scr,verbose,n_min,limit,fpr_factor,max_tries) for x in self.vertices])
 
         #print result
         self.vertices=[r[0] for r in result]
-        
-        for x,par,minlist,score in result:
+        #TODO do we need this line?^
+
+        total_score = 0.0
+        pars ={}
+        subpars ={}
+
+        for x,par,minlist,sc in result:
             pars[x]=par
             subpars[x]=minlist
-            score+=score
+            total_score+=sc
         #print pars
         #move parents from a dictionary to a list
         par_list = []
@@ -608,7 +682,7 @@ class dataset:
             v_signs=self.select_1(v,par).signs()
             for p in par:
                 g.edge_labelling[p,v]=v_signs[p]
-        return score,g,subpars
+        return total_score,g,subpars
    
     def get_stats(self,g):
         stats={}
@@ -850,7 +924,7 @@ class dataset:
         else: (Default)
            output posterior probabilities
         if prob:
-           output only the probability of a sigle class
+           output only the probability of a single class
         """
         f.write("classes")
         for p in self.points:
